# noqa
import argparse
from dodo_commands.extra.standard_commands import DodoCommand
from dodo_commands.framework.util import remove_trailing_dashes
import json
import os


class Command(DodoCommand):  # noqa
    help = ""
    decorators = ["docker"]
    docker_options = [
        '--name=tape',
    ]

    def add_arguments_imp(self, parser):  # noqa
        parser.add_argument(
            'tape_args',
            nargs=argparse.REMAINDER
        )

    def handle_imp(self, tape_args, **kwargs):  # noqa
        stats_file = os.path.join(
            self.get_config("/ROOT/src_dir"),
            "webpack/webpack-stats.json"
        )
        stats = json.loads(open(stats_file).read())
        bundle = os.path.join(
            self.get_config("/VIRT_ROOT/src_dir"),
            "static/bundles",
            stats["chunks"]["main"][0]["name"]
        )

        self.runcmd(
            [
                "tape-run",
                bundle,
            ] + remove_trailing_dashes(tape_args),
            cwd=self.get_config("/TAPE/src_dir")
        )



"""Print the full configuration."""

import yaml
from . import DodoCommand


class Command(DodoCommand):  # noqa
    def handle_imp(self, **kwargs):  # noqa
        print(yaml.dump(self.config, default_flow_style=False, indent=4))



# noqa
import argparse
from dodo_commands.extra.standard_commands import DodoCommand
import os

class Command(DodoCommand):  # noqa
    help = ""
    decorators = ['docker']
    docker_options = [
        '--name=node-sass',
    ]

    def add_arguments_imp(self, parser):  # noqa
        parser.add_argument(
            'nodesass_args',
            nargs=argparse.REMAINDER
        )

    def handle_imp(self, nodesass_args, **kwargs):  # noqa
        nodesass = self.get_config("/SASS/nodesass", "node-sass")
        self.runcmd(
            [
                "mkdir",
                "-p",
                os.path.dirname(self.get_config("/SASS/output_file"))
            ]
        )

        self.runcmd(
            [
                nodesass,
                self.get_config("/SASS/src_file"),
                self.get_config("/SASS/output_file")
            ] + nodesass_args
        )

        self.runcmd(
            [
                nodesass,
                '-w',
                self.get_config("/SASS/src_file"),
                self.get_config("/SASS/output_file")
            ] + nodesass_args
        )



"""Run the webpack command."""
import argparse
from dodo_commands.extra.standard_commands import DodoCommand
from dodo_commands.framework.util import remove_trailing_dashes


class Command(DodoCommand):  # noqa
    decorators = ['docker']
    docker_options = [
        '--name=webpack',
    ]

    def add_arguments_imp(self, parser):  # noqa
        parser.add_argument(
            'webpack_args',
            nargs=argparse.REMAINDER
        )

    def handle_imp(self, webpack_args, **kwargs):  # noqa
        webpack = self.get_config("/WEBPACK/webpack", "webpack")
        webpack_args = webpack_args or ["--watch-stdin"]
        self.runcmd(
            [webpack] + remove_trailing_dashes(webpack_args),
            cwd=self.get_config("/WEBPACK/webpack_dir")
        )



# noqa
from dodo_commands.extra.standard_commands import DodoCommand
import sys


class Command(DodoCommand):  # noqa
    help = ""
    decorators = []

    def add_arguments_imp(self, parser):  # noqa
        parser.add_argument('key')

    def handle_imp(self, key, **kwargs):  # noqa
        sys.stdout.write("%s\n" % str(self.get_config(key, '')))



"""Script for activating/creating a project in the projects dir."""
from dodo_commands.framework.util import create_global_config
import argparse
from argparse import RawTextHelpFormatter
import os
import sys


class DefaultsInstaller:
    """Adds subdirectories to default_commands."""

    @property
    def _extra_dir(self):
        import dodo_commands
        return os.path.join(
            os.path.dirname(dodo_commands.__file__), "extra"
        )

    def _args(self):
        """Get command line args."""
        description = (
            "Install default command directories supplied by the " +
            "paths argument.\nIn addition, you can install the following " +
            "directories that come\nstandard with Dodo Commands:\n\n" +
            ('\n'.join(os.listdir(self._extra_dir)))
        )

        parser = argparse.ArgumentParser(
            description=description, formatter_class=RawTextHelpFormatter
        )
        parser.add_argument(
            "paths",
            nargs='+',
            help='Create symlinks to these command directories'
        )

        return parser.parse_args()

    def _report_error(self, msg):
        sys.stderr.write(msg + os.linesep)

    def _install_commands(self, path):
        """Install the dir with the default commands."""
        dest_dir = os.path.join(
            os.path.expanduser("~/.dodo_commands/default_commands"),
            os.path.basename(path)
        )

        if not os.path.exists(path):
            alt_path = os.path.join(self._extra_dir, path)
            if os.path.exists(alt_path):
                path = alt_path
            else:
                self._report_error("Error: path not found: %s" % path)
                return False

        if os.path.exists(dest_dir):
            self._report_error("%s already exists (no changes were made)." % dest_dir)
            return False

        try:
            os.symlink(os.path.abspath(path), dest_dir)
        except:
            self._report_error("Error: could not create a symlink in %s." % dest_dir)
            return False

        return True

    def run(self):
        """Activate or create a project in the projects dir."""
        create_global_config()

        for path in self._args().paths:
            self._install_commands(path)


def main():  # noqa
    DefaultsInstaller().run()



"""Module for working with the dodo configurations."""

from dodo_commands.framework.command_error import CommandError
from dodo_commands.framework.config_expander import ConfigExpander
from dodo_commands.framework.config_expander import Key, KeyNotFound  # noqa
import glob
import os
import re
import sys
import yaml


def get_project_dir():
    """Return the root dir of the current project."""
    return os.environ['DODO_COMMANDS_PROJECT_DIR']


class ConfigIO:
    """Read and write the dodo configuration."""

    def __init__(self, config_base_dir=None):
        """Arg config_base_dir is where config files are searched.

        Arg config_base_dir defaults to get_project_dir()/dodo_commands/res.
        """
        self.config_base_dir = (
            os.path.join(get_project_dir(), "dodo_commands", "res")
            if config_base_dir is None
            else config_base_dir
        )

    def _path_to(self, post_fix_paths):
        """Return path composed of config_base_dir and post_fix_paths list."""
        return os.path.join(self.config_base_dir, *post_fix_paths)

    def _layers(self, config):
        result = []
        layer_dir = config.get('ROOT', {}).get('layer_dir', '')
        layers = config.get('ROOT', {}).get('layers', [])
        for layer_filename in layers:
            result.append(
                self._path_to([layer_dir, layer_filename])
            )
        return result

    def _add_layer(self, config, layer):
        for section in (layer or {}):
            if section not in config:
                config[section] = layer[section]
                continue
            for key, value in layer[section].items():
                has_key = key in config[section]
                if has_key and isinstance(value, type(dict())):
                    config[section][key].update(value)
                elif has_key and isinstance(value, type(list())):
                    config[section][key].extend(value)
                else:
                    config[section][key] = value

    def load(self, config_filename='config.yaml', load_layers=True):
        """Get configuration."""
        full_config_filename = self._path_to([config_filename])
        if not os.path.exists(full_config_filename):
            return None

        with open(full_config_filename) as f:
            config = yaml.load(f.read())
        if load_layers:
            for layer_filename in self._layers(config):
                if os.path.exists(layer_filename):
                    self._add_layer(config, self.load(layer_filename))
                else:
                    print("Warning: layer not found: %s" % layer_filename)

        return config

    def save(self, config, config_filename='config.yaml'):
        """Write config to config_filename."""
        content = yaml.dump(config, default_flow_style=False, indent=4)
        with open(self._path_to([config_filename]), 'w') as f:
            formatted_content = re.sub(
                r'^([0-9_A-Z]+\:)$',
                r'\n\1',
                content,
                flags=re.MULTILINE
            )
            return f.write(
                # swallow the leading newline
                formatted_content[1:]
                if formatted_content[0] == '\n' else
                formatted_content
            )


def load_dodo_config():
    """Load the project's dodo config and expand it."""
    def _add_to_config(config, section, key, value):
        if section in config:
            if key not in config[section]:
                config[section][key] = value

    config = ConfigIO().load() or dict(ROOT={})
    project_dir = get_project_dir()
    _add_to_config(
        config, "ROOT", "project_name", os.path.basename(project_dir))
    _add_to_config(
        config, "ROOT", "project_dir", project_dir)
    _add_to_config(
        config,
        "ROOT",
        "res_dir",
        os.path.join(project_dir, "dodo_commands", "res"))
    ConfigExpander().run(config)
    return config


class CommandPath:
    """Read search paths for command scripts from the configuration."""

    class Item:  # noqa
        sys_path = ""
        module_path = ""

        @property
        def full_path(self):  # noqa
            return os.path.join(self.sys_path, self.module_path)

    def __init__(self, project_dir, config_base_dir=None):
        """config_base_dir is the directory where config files are searched.

        The config_base_dir arg defaults to project_dir/dodo_commands/res.
        """
        if config_base_dir is None:
            config_base_dir = os.path.join(project_dir, "dodo_commands", "res")

        config = load_dodo_config()
        excluded_dirs = [
            x.full_path for x in self._collect_items(
                config.get('ROOT', {}).get('command_path_exclude', []),
            )
        ]
        self.items = [
            x for x in self._collect_items(
                config.get('ROOT', {}).get('command_path', [])
            )
            if x.full_path not in excluded_dirs
        ]

    def _collect_items(self, patterns):
        items = []
        for pattern in patterns:
            try:
                prefix, postfix = pattern
                prefix = os.path.expanduser(prefix)
            except:
                raise CommandError(
                    "Items in command_path should be of " +
                    "type [<sys-path>, <module-path>]."
                )

            for command_dir in [
                x for x in glob.glob(os.path.join(prefix, postfix))
                if os.path.isdir(x)
            ]:
                item = CommandPath.Item()
                item.sys_path = prefix
                item.module_path = os.path.relpath(command_dir, prefix)
                items.append(item)
        return items

    def extend_sys_path(self):  # noqa
        for item in self.items:
            if item.sys_path not in sys.path:
                sys.path.append(item.sys_path)
